/* message.c
 * 
 * version 0.1
 *
 * last changed : 10/05/2001
 *
 * currently displays the text you give as the first argument on the
 * cmdline
 */

#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

Window CreateMessageWindow (Display* dpy, int screen_num)
{
    int vmask;
    XSetWindowAttributes attribs;
    Window msg_w;

    /* message window */
    vmask = /*CWBackPixel|*/CWSaveUnder|CWOverrideRedirect|CWColormap
        |CWBorderPixel;
    attribs.save_under = True;
    attribs.override_redirect = True;
    attribs.colormap = DefaultColormap (dpy, screen_num);
    attribs.background_pixel = 1000;
/*    attribs.background_pixel = scr->icon_back_texture->normal.pixel;  */
    attribs.border_pixel = 0; /* do not care */


    msg_w =
      XCreateWindow(dpy, DefaultRootWindow (dpy), 0, 0, 800, 30, 0,
CopyFromParent,
                    CopyFromParent, CopyFromParent, vmask, &attribs);

//    msg_w = DefaultRootWindow (dpy);
/*
  msg_w = XCreateSimpleWindow(dpy, RootWindow(dpy, screen_num),
                            0, 0, 10, 10, 0,
                            BlackPixel(dpy, screen_num),
                            WhitePixel(dpy, screen_num));
*/
    XMapWindow (dpy, msg_w);
    return msg_w;
}

GC
create_gc(Display* display, Window win, int reverse_video)
{
  GC gc;                                /* handle of newly created GC.  */
  unsigned long valuemask = 0;          /* which values in 'values'  to */
                                        /* check when creating the GC.  */
                                       /* check when creating the GC.  */
  XGCValues values;                     /* initial values for the GC.   */
  unsigned int line_width = 2;          /* line width for the GC.       */
  int line_style = LineSolid;           /* style for lines drawing and  */
  int cap_style = CapButt;              /* style of the line's edje and */
  int join_style = JoinBevel;           /*  joined lines.               */
  int screen_num = DefaultScreen(display);

  gc = XCreateGC(display, win, valuemask, &values);
  if (gc < 0) {
        fprintf(stderr, "XCreateGC: \n");
  }
  XSetSubwindowMode ( display, gc, IncludeInferiors );
  /* allocate foreground and background colors for this GC. */
  if (reverse_video) {
    XSetForeground(display, gc, WhitePixel(display, screen_num));
    XSetBackground(display, gc, BlackPixel(display, screen_num));
  }
  else {
    XSetForeground(display, gc, BlackPixel(display, screen_num));
    XSetBackground(display, gc, WhitePixel(display, screen_num));
  }
  
  /* define the style of lines that will be drawn using this GC. */
  XSetLineAttributes(display, gc,
                     line_width, line_style, cap_style, join_style);
  
  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle(display, gc, FillSolid);

  return gc;
}

int main (int argc, char *argv[])
{
  Display* display;             /* pointer to X Display structure.           */
  Window msg_w;

  char *display_name = getenv("DISPLAY");  /* address of the X display.      */

  int screen_num;               /* number of screen to place the window on.  */
  unsigned int display_width,
               display_height;  /* height and width of the X display.        */

  GC gc;                        /* GC (graphics context) used for drawing */
                                /*  in our window.                           */
  XFontStruct* font_info;       /* Font structure, used for drawing text.    */
  char* font_name = "-*-helvetica-bold-r-normal-*-24-*-*-*-*-*-*-*"; /*
font to use for drawing text.   */

  /* open connection with the X server. */
  display = XOpenDisplay(display_name);
  if (display == NULL) {
    fprintf(stderr, "%s: cannot connect to X server '%s'\n",
            argv[0], display_name);
    exit(1);
  }
  
  /* get the geometry of the default screen for our display. */
  screen_num = DefaultScreen(display);
  display_width = DisplayWidth(display, screen_num);
  display_height = DisplayHeight(display, screen_num);

  msg_w = CreateMessageWindow (display, screen_num);

  /* allocate a new GC (graphics context) for drawing in the window. */

  gc = create_gc(display, msg_w, 0);
  XSync(display, False);

  /* try to load the given font. */
  font_info = XLoadQueryFont(display, font_name);
  if (!font_info) {
      fprintf(stderr, "XLoadQueryFont: failed loading font '%s'\n",font_name);
      exit(-1);
  }

  /* assign the given font to our GC. */
  XSetFont(display, gc, font_info->fid);
//  XSetFont(display, reverse_gc, font_info->fid);

  {
    /* variables used for drawing the text strings. */
    int x, y;
    char* text_string;  
    //int string_width;
    int font_height;
  
    /* find the height of the characters drawn using this font.        */
    font_height = font_info->ascent + font_info->descent;
    /* draw a "hello world" string on the top-left side of our window. */
    text_string = "byebye world";
    x = 0;   
    y = font_height;    
    XDrawString(display, msg_w, gc, x, y, argv[1],
strlen(argv[1]));
    /* flush all pending requests to the X server. */
    XFlush(display);      
  }

  sleep(4);

  /* close the connection to the X server. */
  XCloseDisplay(display);

  return 1;
}
